/* ggisyna - program to display sound graphically
   Copyright (C) 1997       Paul Francis Harrison
   Copyright (C) 1997-1998  Marcus Sundberg

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "syna.h"

static double cosTable[SAMPLES], sinTable[SAMPLES];
static int bitReverse[SAMPLES];
static short spldata[SAMPLES*2];

int
bitReverser(int i) {
    int sum=0,j;
    for(j=0;j<m;j++) {
        sum = (i&1)+sum*2;
        i >>= 1;
    }
    return sum;
}

void
fft(double *x,double *y) {
    int n2 = SAMPLES, n1;
    int twoToTheK;
    for(twoToTheK=1;twoToTheK<SAMPLES;twoToTheK*=2) {
	int j;
	n1 = n2;
	n2 /= 2;
	for(j=0;j<n2;j++) {
	    int i;
	    double c = cosTable[j*twoToTheK%SAMPLES], s = -sinTable[j*twoToTheK%SAMPLES];
	    for(i=j;i<SAMPLES;i+=n1) {
		int l = i+n2;
		double xt = x[i] - x[l];
		double yt = y[i] - y[l];
		x[i] = (x[i] + x[l]);
		y[i] = (y[i] + y[l]);
		x[l] = xt*c - yt*s;
		y[l] = xt*s + yt*c;
	    }
	}
    }
}


void
do_precalcs(void) {
    int i;
    for(i=0;i<SAMPLES;i++) {
        sinTable[i] = sin(3.141592*2/SAMPLES*i);
        cosTable[i] = cos(3.141592*2/SAMPLES*i);
        bitReverse[i] = bitReverser(i);
    }
#define BOUND(x) ((x) > 63 ? 63 : (x))
#define PEAKIFY(x) BOUND((x) - (x)*(63-(x))/63/2)
    for(i=0;i<256;i++)
        setPalette(i,PEAKIFY((i&15)*4),
                   PEAKIFY((i&15*16)/4+(i&15)),
                   PEAKIFY((i&15*16)/4));
}


void
syna_loop(void) {
#ifdef SLOW_MEM
    unsigned int *ptr;
#else
    unsigned char *ptr;
#endif
    double x[SAMPLES], y[SAMPLES];
    double a[SAMPLES], b[SAMPLES];
    int clarity[SAMPLES]; /* Surround sound */
    int hmul=((width/320.0)*(height/200.0))*320;
    int pixels=width*height;
    int i;
    
    memset(backbuf,0,pixels*2*bpp);
    
    for(;;) {
        if (processUserInput()) break;
        
        getNextFragment(spldata);
        
        for(i=0;i<SAMPLES;i++) {
            x[i] = spldata[i*2];
            y[i] = spldata[i*2+1];
        }
        fft(x,y);
        for(i=0 +1;i<SAMPLES;i++) {
            double x1 = x[bitReverse[i]], 
                y1 = y[bitReverse[i]], 
                x2 = x[bitReverse[SAMPLES-i]], 
                y2 = y[bitReverse[SAMPLES-i]],
                aa,bb;
            a[i] = sqrt(aa= (x1+x2)*(x1+x2) + (y1-y2)*(y1-y2) );
            b[i] = sqrt(bb= (x1-x2)*(x1-x2) + (y1+y2)*(y1+y2) );
            clarity[i] = (int)(
                               ( (x1+x2) * (x1-x2) + (y1+y2) * (y1-y2) )/(aa+bb) * 256 );
        } 
        
        ptr = (void*)backbuf;
        i = 2*pixels/sizeof(*ptr);
        do {
#ifdef SLOW_MEM
#define HIGH4MASK 0x0f0f0f0f
            *(ptr++) -= ( ((((unsigned char*)ptr)[0]
                             +(((unsigned char*)ptr)[0]>>1))>>4)
                           |((((unsigned char*)ptr)[1]
                              +(((unsigned char*)ptr)[1]>>1))>>4<<8)
                           |((((unsigned char*)ptr)[2]
                              +(((unsigned char*)ptr)[2]>>1))>>4<<16)
                           |((((unsigned char*)ptr)[3]
                              +(((unsigned char*)ptr)[3]>>1))>>4<<24) );
#else
            *(ptr++) -= (*ptr+(*ptr>>1))>>4;
#endif
        } while(--i != 0);
        
        for(i=1;i<SAMPLES/2;i++) {
            int h = (int)( b[i]*(width-40) / (a[i]+b[i]+0.0001)+20 );
            if (h >= 10 && h < width-10) {
                unsigned char *p;
                int x;
                int br = (int)((a[i]+b[i])*i*(brightness/65536.0/SAMPLES));
                int br1 = br*(clarity[i]+128)>>8;
                int br2 = br*(128-clarity[i])>>8;
                if (br1 < 0) br1 = 0; else if (br1 > 255) br1 = 255;
                if (br2 < 0) br2 = 0; else if (br2 > 255) br2 = 255;
                p = backbuf + h+(164-((i<<8)>>m))*hmul;
                
#define doit(o) {if (p[o] < 255-br1) p[o] += br1; else p[o] = 255; \
                    if (p[o+1] < 255-br2) p[o+pixels] += br2; else p[o+1] = 255;}
                doit(0);
                for (x=1;br1 > 16 || br2 > 16;x+=1,
                         br1 = br1*200>>8,
                         br2 = br2*200>>8) {
                    doit(x);
                    doit(-x);
                    doit(x*width);
                    doit(-(x*width));
                }
            }
        }
        
        showOutput();
    }
    
}
