/* ggisyna - program to display sound graphically
   Copyright (C) 1997       Paul Francis Harrison
   Copyright (C) 1997-1998  Marcus Sundberg

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <linux/cdrom.h>

#include "syna.h"

static int cdDevice = -1;
static int trackcount = 0;
static int ctrack = 0, cpaused = 0, cplaying = 0;
static struct timeval lasttime = {0, 0};

static int
cdNeedUpdate(void)
{
	struct timeval curtim;

	gettimeofday(&curtim, NULL);
	if (curtim.tv_sec - lasttime.tv_sec >= 2) {
		lasttime = curtim;
		return 1;
	}
	
	return 0;
}	

static int
cdOpen(int block) {
	if (cdDevice != -1) {
		return 1;
	}
	if ((cdDevice = open(CDDEV, O_RDONLY | (block ? O_NONBLOCK : 0)))
	    < 0) {
		return 0;
	}

	return cdDevice;
}

static void
cdClose(void) {
	close(cdDevice);
	cdDevice = -1;
}

int
cdGetTrackCount(void) {
    struct cdrom_tochdr cdTochdr;
    
    if (cdNeedUpdate() || !trackcount) {
	    if (!cdOpen(0)) return 0;
		
	    attemptNoDie(ioctl(cdDevice, CDROMREADTOCHDR, &cdTochdr),
			 "reading cd track info");
	    cdClose();
	    trackcount = cdTochdr.cdth_trk1;
    }

    return trackcount;
}

void
cdPlay(int track) {
	struct cdrom_ti ti;

	ti.cdti_trk0 = track;
	ti.cdti_ind0 = 1;
	ti.cdti_trk1 = cdGetTrackCount();
	ti.cdti_ind1 = 99;
	if (!cdOpen(1)) return;
	while(ti.cdti_trk0 <= ti.cdti_trk1
	      && -1 == ioctl(cdDevice, CDROMPLAYTRKIND, &ti))
		ti.cdti_trk0++; //Skip over data tracks
	cdClose();
}


void
cdPause(void) {
	if (!cdOpen(1)) return;
	attemptNoDie(ioctl(cdDevice, CDROMPAUSE),"pausing cd");
	cdClose();
}


void
cdResume(void) {
	if (!cdOpen(1)) return;
	attemptNoDie(ioctl(cdDevice, CDROMRESUME),"resuming cd");
	cdClose();
}


void
cdStop(void) {
	if (!cdOpen(1)) return;
	attemptNoDie(ioctl(cdDevice, CDROMSTOP),"stopping cd");
	cdClose();
}


void
cdGetStatus(int *track, int *paused, int *playing) {
	if (cdNeedUpdate() || ctrack == 0) {
		if (cdOpen(0)) { 
			struct cdrom_subchnl subchnl;
			subchnl.cdsc_format = CDROM_MSF;
			attemptNoDie(ioctl(cdDevice, CDROMSUBCHNL, &subchnl),
				     "querying the cd");
			ctrack = subchnl.cdsc_trk;
			cpaused = (subchnl.cdsc_audiostatus
				   == CDROM_AUDIO_PAUSED);
			cplaying = (subchnl.cdsc_audiostatus
				    == CDROM_AUDIO_PLAY);
			if (!cpaused && !cplaying) ctrack = 1;
			cdClose();
		} else {
			ctrack = 1;
			cpaused = 0;
			cplaying = 0;
		}
	}
	*track = ctrack;
	*paused = cpaused;
	*playing = cplaying;
}
