/* ggisyna - program to display sound graphically
   Copyright (C) 1997       Paul Francis Harrison
   Copyright (C) 1997-1998  Marcus Sundberg

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <fcntl.h>



#include "syna.h"
#include "alphabet.h"


static int pipes[2];
static int fragsLost = 0, fragsGot = 0;

char *PROGNAME;
int source=SYNSND_NONE;
int width=320, height=200;


int
main(int argc, char **argv) {
    int configPlayTrack = -1;
    
    PROGNAME = argv[0];
    switch (argc) {
    case 4:
        height=atoi(argv[3]);
        if (height%200 || height == 0) {
            error("height must be an integer multiple of 200");
        }
    case 3:
        width=atoi(argv[2]);
        if (width%4 || width == 0) {
            error("width must be an integer multiple of 4");
        }
    case 2:
        break;
    case 1:
    default:
        fprintf(stderr, "\nggisyna v" SYNAVERSION "\n\n"
                "Usage: %s input [width [height]]\n\n"
                "  Where input is one of:\n"
                "    cd       - listen to a CD\n"
                "    mic      - listen to microphone input\n"
                "    line     - listen to line input\n"
                "    synth    - listen to synth input\n"
                "    <track>  - listen to this CD track\n\n",
                PROGNAME);
        return 1;
    }
    
    if (strcmp(argv[1],"line") == 0) {
        source = SYNSND_LINE;
    } else if (strcmp(argv[1],"mic") == 0) {
        source = SYNSND_MIC;
    } else if (strcmp(argv[1],"synth") == 0) {
        source = SYNSND_SYNTH;
    } else if (strcmp(argv[1],"cd") == 0
             || sscanf(argv[1],"%i",&configPlayTrack) == 1) {
        source = SYNSND_CD;
    } else 
        error("comprehending user's bizzare requests");
    
    attempt(pipe(pipes),"opening pipe");
    
    sndSource(source);
    
    if (configPlayTrack != -1)
        cdPlay(configPlayTrack);    
        
    gfxOpen();

    do_precalcs();
    
    attempt(sndStart(pipes),"Starting sound");

    close(pipes[1]);
    
    syna_loop();
    
    close(pipes[0]);

    gfxClose();

    if (paused)
        cdStop();    
    
    printf("Frame rate: %.1f frames/second\n",
           (double)(FREQUENCY)/SAMPLES * fragsGot/(fragsLost+fragsGot));
    if (fragsLost > fragsGot)
        printf("Fragments: %i lost as compared to %i processed.\n"
               "Your computer may not be fast enough to give a good frame rate.\n",
               fragsLost,fragsGot);
    fflush(stdout);
               
    return 0;
}


void
error(char *str) { 
    fprintf(stderr, "%s: Error: %s\n",PROGNAME,str); 
    exit(1);
}

void
warning(char *str) {
    fprintf(stderr, "%s: Non-fatal error %s\n",PROGNAME,str);
}

void
putIcon(int ox,int oy,int id,int channel) {
    int i,x,y, any;
    unsigned char *alpha = alphabet, *dest = backbuf+ox+oy*width+channel;

    for(i=0;i<id;i++) {
        do for(x=0,any=0;x<32;x++,alpha++) any = (any || *alpha > 'a'+2);
        while(!any);
        do for(x=0,any=0;x<32;x++,alpha++) any = (any || *alpha > 'a'+2);
        while(any);
    }
    do for(x=0,any=0;x<32;x++,alpha++) any = (any || *alpha > 'a'+2);
    while(!any);
    alpha -= 32;

    any = 1;
    for(x=0;any;x++)
        for(y=0,any=0;y<32;y++,alpha++)
            if (*alpha > 'a'+2) {
                int o = x+y*width;
                if (dest[o] < (*alpha-'a')*16)
                    dest[o] = (unsigned char)(*alpha-'a')*16;
                any = 1;
            }
}


void
getNextFragment(void *data) {
    fcntl(pipes[0],F_SETFL,0);
    while(read(pipes[0],data,4*SAMPLES) != 4*SAMPLES);
    fcntl(pipes[0],F_SETFL,O_NONBLOCK);
    while(read(pipes[0],data,4*SAMPLES) == 4*SAMPLES) fragsLost ++;
    fragsGot++;
}
